@@ -1,4 +1,3 @@
-
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
@@ -15,6 +14,8 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
+RCSID
+    ("$Header: /cvs/openafs/src/afs/UKERNEL/afs_usrops.c,v 1.30.6.6 2007/11/26 21:08:40 shadow Exp $");
 
 
 #ifdef	UKERNEL
@@ -55,7 +56,7 @@ char afs_LclCellName[64];
 
 struct usr_vnode *afs_FileTable[MAX_OSI_FILES];
 int afs_FileFlags[MAX_OSI_FILES];
-off_t afs_FileOffsets[MAX_OSI_FILES];
+int afs_FileOffsets[MAX_OSI_FILES];
 
 #define MAX_CACHE_LOOPS 4
 
@@ -106,8 +107,8 @@ int usr_udpcksum = 0;
 
 usr_key_t afs_global_u_key;
 
-struct usr_proc *afs_global_procp = NULL;
-struct usr_ucred *afs_global_ucredp = NULL;
+struct usr_proc *afs_global_procp;
+struct usr_ucred *afs_global_ucredp;
 struct usr_sysent usr_sysent[200];
 
 #ifdef AFS_USR_OSF_ENV
@@ -986,10 +987,9 @@ shutdown_osifile(void)
     return;
 }
 
-int
+void
 afs_nfsclient_init(void)
 {
-    return 0;
 }
 
 void
@@ -1004,12 +1004,6 @@ afs_osi_Invisible(void)
     return;
 }
 
-void
-afs_osi_Visible(void)
-{
-    return;
-}
-
 int
 osi_GetTime(struct timeval *tv)
 {
@@ -1314,13 +1308,8 @@ SweepAFSCache(int *vFilesFound)
     for (currp = readdir(cdirp); currp; currp = readdir(cdirp)) {
 	if (afsd_debug) {
 	    printf("%s: Current directory entry:\n", rn);
-#if defined(AFS_USR_DFBSD_ENV)
-	    printf("\tinode=%d, name='%s'\n", currp->d_ino,
-		   currp->d_name);
-#else
 	    printf("\tinode=%d, reclen=%d, name='%s'\n", currp->d_ino,
 		   currp->d_reclen, currp->d_name);
-#endif
 	}
 
 	/*
@@ -1566,7 +1555,7 @@ uafs_Init(char *rn, char *mountDirParam, char *confDirParam,
 	cacheStatEntries = cacheStatEntriesParam;
     }
     strcpy(cacheBaseDir, cacheBaseDirParam);
-    if (nDaemonsParam != 0) {
+    if (nDaemons != 0) {
 	nDaemons = nDaemonsParam;
     } else {
 	nDaemons = 3;
@@ -1645,7 +1634,7 @@ uafs_Init(char *rn, char *mountDirParam, char *confDirParam,
     /*
      * Set the primary cell name.
      */
-    call_syscall(AFSCALL_CALL, AFSOP_SET_THISCELL, (long)afs_LclCellName, 0, 0, 0);
+    call_syscall(AFSOP_SET_THISCELL, (long)afs_LclCellName, 0, 0, 0, 0);
 
     if ((logfd = fopen(fullpn_AFSLogFile, "r+")) == 0) {
 	if (afsd_verbose)
@@ -1851,11 +1840,7 @@ uafs_Init(char *rn, char *mountDirParam, char *confDirParam,
 			 (long)pathname_for_V[currVFile], 0, 0, 0);
 	}
     /*end for */
-/*#ifndef NETSCAPE_NSAPI*/
-#if 0
-/* this breaks solaris if the kernel-mode client has never been installed,
- * and it doesn't seem to work now anyway, so just disable it */
-
+#ifndef NETSCAPE_NSAPI
     /*
      * Copy our tokens from the kernel to the user space client
      */
@@ -2071,7 +2056,6 @@ uafs_SetTokens(char *tbuffer, int tlen)
     iob.in_size = tlen;
     iob.out = &outbuf[0];
     iob.out_size = sizeof(outbuf);
-
     rc = call_syscall(AFSCALL_PIOCTL, 0, _VICEIOCTL(3), (long)&iob, 0, 0);
     if (rc != 0) {
 	errno = rc;
@@ -2211,7 +2195,7 @@ int
 uafs_LookupName(char *path, struct usr_vnode *parentVp,
 		struct usr_vnode **vpp, int follow, int no_eval_mtpt)
 {
-    int code = 0;
+    int code;
     int linkCount;
     struct usr_vnode *vp;
     struct usr_vnode *nextVp;
@@ -2701,7 +2685,6 @@ uafs_open_r(char *path, int flags, int mode)
 		errno = code;
 		return -1;
 	    }
-	    fileP = AFSTOV(vc);
 	} else {
 	    fileP = NULL;
 	    code = uafs_LookupName(nameP, dirP, &fileP, 1, 0);
@@ -2774,7 +2757,6 @@ uafs_open_r(char *path, int flags, int mode)
      */
     if ((flags & O_TRUNC) && (attrs.va_size != 0)) {
 	usr_vattr_null(&attrs);
-	attrs.va_mask = ATTR_SIZE;
 	attrs.va_size = 0;
 	code = afs_setattr(VTOAFS(fileP), &attrs, u.u_cred);
 	if (code != 0) {
@@ -2846,23 +2828,13 @@ uafs_write(int fd, char *buf, int len)
 {
     int retval;
     AFS_GLOCK();
-    retval = uafs_pwrite_r(fd, buf, len, afs_FileOffsets[fd]);
+    retval = uafs_write_r(fd, buf, len);
     AFS_GUNLOCK();
     return retval;
 }
 
 int
-uafs_pwrite(int fd, char *buf, int len, off_t offset)
-{
-    int retval;
-    AFS_GLOCK();
-    retval = uafs_pwrite_r(fd, buf, len, offset);
-    AFS_GUNLOCK();
-    return retval;
-}
-
-int
-uafs_pwrite_r(int fd, char *buf, int len, off_t offset)
+uafs_write_r(int fd, char *buf, int len)
 {
     int code;
     struct usr_uio uio;
@@ -2885,7 +2857,7 @@ uafs_pwrite_r(int fd, char *buf, int len, off_t offset)
     iov[0].iov_len = len;
     uio.uio_iov = &iov[0];
     uio.uio_iovcnt = 1;
-    uio.uio_offset = offset;
+    uio.uio_offset = afs_FileOffsets[fd];
     uio.uio_segflg = 0;
     uio.uio_fmode = FWRITE;
     uio.uio_resid = len;
@@ -2912,23 +2884,13 @@ uafs_read(int fd, char *buf, int len)
 {
     int retval;
     AFS_GLOCK();
-    retval = uafs_pread_r(fd, buf, len, afs_FileOffsets[fd]);
+    retval = uafs_read_r(fd, buf, len);
     AFS_GUNLOCK();
     return retval;
 }
 
 int
-uafs_pread(int fd, char *buf, int len, off_t offset)
-{
-    int retval;
-    AFS_GLOCK();
-    retval = uafs_pread_r(fd, buf, len, offset);
-    AFS_GUNLOCK();
-    return retval;
-}
-
-int
-uafs_pread_r(int fd, char *buf, int len, off_t offset)
+uafs_read_r(int fd, char *buf, int len)
 {
     int code;
     struct usr_uio uio;
@@ -2952,7 +2914,7 @@ uafs_pread_r(int fd, char *buf, int len, off_t offset)
     iov[0].iov_len = len;
     uio.uio_iov = &iov[0];
     uio.uio_iovcnt = 1;
-    uio.uio_offset = offset;
+    uio.uio_offset = afs_FileOffsets[fd];
     uio.uio_segflg = 0;
     uio.uio_fmode = FREAD;
     uio.uio_resid = len;
@@ -3136,7 +3098,6 @@ uafs_chmod_r(char *path, int mode)
 	return -1;
     }
     usr_vattr_null(&attrs);
-    attrs.va_mask = ATTR_MODE;
     attrs.va_mode = mode;
     code = afs_setattr(VTOAFS(vp), &attrs, u.u_cred);
     VN_RELE(vp);
@@ -3173,7 +3134,6 @@ uafs_fchmod_r(int fd, int mode)
 	return -1;
     }
     usr_vattr_null(&attrs);
-    attrs.va_mask = ATTR_MODE;
     attrs.va_mode = mode;
     code = afs_setattr(VTOAFS(vp), &attrs, u.u_cred);
     if (code != 0) {
@@ -3209,7 +3169,6 @@ uafs_truncate_r(char *path, int length)
 	return -1;
     }
     usr_vattr_null(&attrs);
-    attrs.va_mask = ATTR_SIZE;
     attrs.va_size = length;
     code = afs_setattr(VTOAFS(vp), &attrs, u.u_cred);
     VN_RELE(vp);
@@ -3246,7 +3205,6 @@ uafs_ftruncate_r(int fd, int length)
 	return -1;
     }
     usr_vattr_null(&attrs);
-    attrs.va_mask = ATTR_SIZE;
     attrs.va_size = length;
     code = afs_setattr(VTOAFS(vp), &attrs, u.u_cred);
     if (code != 0) {
