@@ -24,7 +24,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_attrs.c,v 1.27.2.3 2004/12/07 06:12:13 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_attrs.c,v 1.41.2.1 2006/11/10 00:11:08 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -51,17 +51,12 @@ afs_CopyOutAttrs(register struct vcache *avc, register struct vattr *attrs)
     AFS_STATCNT(afs_CopyOutAttrs);
     if (afs_fakestat_enable && avc->mvstat == 1)
 	fakedir = 1;
-#if	defined(AFS_MACH_ENV )
-    attrs->va_mode =
-	fakedir ? VDIR | 0755 : vType(avc) | (avc->m.Mode & ~VFMT);
-#else /* AFS_MACH_ENV */
     attrs->va_type = fakedir ? VDIR : vType(avc);
 #if defined(AFS_SGI_ENV) || defined(AFS_AIX32_ENV) || defined(AFS_SUN5_ENV)
     attrs->va_mode = fakedir ? 0755 : (mode_t) (avc->m.Mode & 0xffff);
 #else
     attrs->va_mode = fakedir ? VDIR | 0755 : avc->m.Mode;
 #endif
-#endif /* AFS_MACH_ENV */
 
     if (avc->m.Mode & (VSUID | VSGID)) {
 	/* setuid or setgid, make sure we're allowed to run them from this cell */
@@ -94,37 +89,33 @@ afs_CopyOutAttrs(register struct vcache *avc, register struct vattr *attrs)
 #endif /* AFS_DARWIN_ENV */
     attrs->va_uid = fakedir ? 0 : avc->m.Owner;
     attrs->va_gid = fakedir ? 0 : avc->m.Group;	/* yeah! */
-#if	defined(AFS_SUN56_ENV)
+#if defined(AFS_SUN56_ENV)
     attrs->va_fsid = avc->v.v_vfsp->vfs_fsid.val[0];
-#else
-#ifdef	AFS_SUN5_ENV
-    /* XXX We try making this match the vfs's dev field  XXX */
-    attrs->va_fsid = 1;
-#else
-#ifdef AFS_OSF_ENV
+#elif defined(AFS_OSF_ENV)
     attrs->va_fsid = avc->v.v_mount->m_stat.f_fsid.val[0];
-#else
-#ifdef AFS_DARWIN70_ENV
-    attrs->va_fsid = avc->v.v_mount->mnt_stat.f_fsid.val[0];
+#elif defined(AFS_DARWIN80_ENV)
+    VATTR_RETURN(attrs, va_fsid, vfs_statfs(vnode_mount(AFSTOV(avc)))->f_fsid.val[0]);
+#elif defined(AFS_DARWIN70_ENV)
+    attrs->va_fsid = avc->v->v_mount->mnt_stat.f_fsid.val[0];
 #else /* ! AFS_DARWIN70_ENV */
     attrs->va_fsid = 1;
-#endif /* AFS_DARWIN70_ENV */
-#endif
-#endif
-#endif /* AFS_SUN56_ENV */
+#endif 
     if (avc->mvstat == 2) {
 	tvp = afs_GetVolume(&avc->fid, 0, READ_LOCK);
 	/* The mount point's vnode. */
 	if (tvp) {
 	    attrs->va_nodeid =
-		tvp->mtpoint.Fid.Vnode + (tvp->mtpoint.Fid.Volume << 16);
+	      afs_calc_inum (tvp->mtpoint.Fid.Volume,
+			      tvp->mtpoint.Fid.Vnode);
 	    if (FidCmp(&afs_rootFid, &avc->fid) && !attrs->va_nodeid)
 		attrs->va_nodeid = 2;
 	    afs_PutVolume(tvp, READ_LOCK);
 	} else
 	    attrs->va_nodeid = 2;
     } else
-	attrs->va_nodeid = avc->fid.Fid.Vnode + (avc->fid.Fid.Volume << 16);
+	attrs->va_nodeid = 
+	      afs_calc_inum (avc->fid.Fid.Volume,
+			      avc->fid.Fid.Vnode);
     attrs->va_nodeid &= 0x7fffffff;	/* Saber C hates negative inode #s! */
     attrs->va_nlink = fakedir ? 100 : avc->m.LinkCount;
     attrs->va_size = fakedir ? 4096 : avc->m.Length;
@@ -156,22 +147,11 @@ afs_CopyOutAttrs(register struct vcache *avc, register struct vattr *attrs)
     attrs->va_flags = 0;
 #endif
 #if defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)
-    attrs->va_blksize = PAGESIZE;	/* XXX Was 8192 XXX */
+    attrs->va_blksize = AFS_BLKSIZE;	/* XXX Was 8192 XXX */
 #else
-    attrs->va_blocksize = PAGESIZE;	/* XXX Was 8192 XXX */
+    attrs->va_blocksize = AFS_BLKSIZE;	/* XXX Was 8192 XXX */
 #endif
-#ifdef AFS_DEC_ENV
-    /* Have to use real device #s in Ultrix, since that's how FS type is
-     * encoded.  If rdev doesn't match Ultrix equivalent of statfs's rdev, then
-     * "df ." doesn't work.
-     */
-    if (afs_globalVFS && afs_globalVFS->vfs_data)
-	attrs->va_rdev = ((struct mount *)(afs_globalVFS->vfs_data))->m_dev;
-    else
-	attrs->va_rdev = 1;	/* better than nothing */
-#else
     attrs->va_rdev = 1;
-#endif
 #if defined(AFS_HPUX110_ENV)
     if (afs_globalVFS)
 	attrs->va_fstype = afs_globalVFS->vfs_mtype;
@@ -181,47 +161,19 @@ afs_CopyOutAttrs(register struct vcache *avc, register struct vattr *attrs)
      * Below return 0 (and not 1) blocks if the file is zero length. This conforms
      * better with the other filesystems that do return 0.      
      */
-#if !defined(AFS_OSF_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_XBSD_ENV)
-#if !defined(AFS_HPUX_ENV)
-#ifdef	AFS_SUN5_ENV
-    attrs->va_nblocks =
-	(attrs->va_size ? ((attrs->va_size + 1023) >> 10) << 1 : 0);
-#elif defined(AFS_SGI_ENV)
-    attrs->va_blocks = BTOBB(attrs->va_size);
-#else
-    attrs->va_blocks =
-	(attrs->va_size ? ((attrs->va_size + 1023) >> 10) << 1 : 0);
-#endif
-#else /* !defined(AFS_HPUX_ENV) */
-    attrs->va_blocks = (attrs->va_size ? ((attrs->va_size + 1023) >> 10) : 0);
-#endif /* !defined(AFS_HPUX_ENV) */
-#else /* ! AFS_OSF_ENV && !AFS_DARWIN_ENV && !AFS_XBSD_ENV */
+#if defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
     attrs->va_bytes = (attrs->va_size ? (attrs->va_size + 1023) : 1024);
 #ifdef	va_bytes_rsv
     attrs->va_bytes_rsv = -1;
 #endif
-#endif /* ! AFS_OSF_ENV && !AFS_DARWIN_ENV && !AFS_XBSD_ENV */
-
-#ifdef AFS_LINUX22_ENV
-    /* And linux has its own stash as well. */
-    vattr2inode(AFSTOV(avc), attrs);
-#endif
-#ifdef notdef
-#ifdef AFS_AIX51_ENV
-    afs_Trace2(afs_iclSetp, CM_TRACE_STATACLX, ICL_TYPE_POINTER,
-	       attrs->va_acl, ICL_TYPE_INT32, attrs->va_aclsiz);
-    if (attrs->va_acl && attrs->va_aclsiz >= 12) {
-	struct acl *ap;
-
-	ap = (struct acl *)attrs->va_acl;
-	ap->acl_len = 8;
-	ap->acl_mode = ACL_MODE;
-	ap->acl_rsvd = 0;
-	ap->u_access = 7;
-    }
-    /* temporary fix ? */
-    attrs->va_aclsiz = 1;
-#endif
+#elif defined(AFS_HPUX_ENV)
+    attrs->va_blocks = (attrs->va_size ? ((attrs->va_size + 1023)>>10) : 0);
+#elif defined(AFS_SGI_ENV)
+    attrs->va_blocks = BTOBB(attrs->va_size);
+#elif defined(AFS_SUN5_ENV)
+    attrs->va_nblocks = (attrs->va_size ? ((attrs->va_size + 1023)>>10)<<1:0);
+#else /* everything else */
+    attrs->va_blocks = (attrs->va_size ? ((attrs->va_size + 1023)>>10)<<1:0);
 #endif
     return 0;
 }
@@ -271,14 +223,14 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, struct AFS_UCRED *acred)
 	return code;
     }
 #endif
-#if defined(AFS_DARWIN_ENV)
+#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN80_ENV)
     if (avc->states & CUBCinit) {
 	code = afs_CopyOutAttrs(avc, attrs);
 	return code;
     }
 #endif
 
-#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
     afs_BozonLock(&avc->pvnLock, avc);
 #endif
 
@@ -293,7 +245,7 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, struct AFS_UCRED *acred)
     } else
 	code = 0;
 
-#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
     if (code == 0)
 	osi_FlushPages(avc, acred);
     afs_BozonUnlock(&avc->pvnLock, avc);
@@ -317,22 +269,6 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, struct AFS_UCRED *acred)
 				     CMB_ALLOW_EXEC_AS_READ)) {
 		    return EACCES;
 		}
-#if 0
-/* The effect of the following is to force the NFS client to refetch the
- * volume root every time, since the mtime changes.  For Solaris 9 NFSv3
- * clients, this means looping forever, since for some reason (related
- * to caching?) it wants the mtime to be consistent two reads in a row.
- * Why are volume roots special???
- * --jhutz 2-May-2004
- */
-		if (avc->mvstat == 2) {
-#if defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV) || defined(AFS_AIX41_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-		    attrs->va_mtime.tv_nsec += ((++avc->xlatordv) * 1000);
-#else
-		    attrs->va_mtime.tv_usec += ++avc->xlatordv;
-#endif
-		}
-#endif
 	    }
 	    if ((au = afs_FindUser(treq.uid, -1, READ_LOCK))) {
 		register struct afs_exporter *exporter = au->exporter;
@@ -355,13 +291,23 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, struct AFS_UCRED *acred)
 		     * confuses getwd()... */
 #ifdef AFS_LINUX22_ENV
 		    if (avc == afs_globalVp) {
-			struct inode *ip = avc->v.i_sb->s_root->d_inode;
-			attrs->va_nodeid = ip->i_ino;
+			struct inode *ip = AFSTOV(avc)->i_sb->s_root->d_inode;
+			attrs->va_nodeid = ip->i_ino;	/* VTOI()? */
 		    }
 #else
-		    if (AFSTOV(avc)->v_flag & VROOT) {
+		    if (
+#ifdef AFS_DARWIN_ENV		    
+			vnode_isvroot(AFSTOV(avc))
+#else
+			AFSTOV(avc)->v_flag & VROOT
+#endif
+			) {
 			struct vnode *vp = AFSTOV(avc);
 
+#ifdef AFS_DARWIN80_ENV
+			/* XXX vp = vnode_mount(vp)->mnt_vnodecovered; */
+			vp = 0;
+#else
 			vp = vp->v_vfsp->vfs_vnodecovered;
 			if (vp) {	/* Ignore weird failures */
 #ifdef AFS_SGI62_ENV
@@ -374,6 +320,7 @@ afs_getattr(OSI_VC_DECL(avc), struct vattr *attrs, struct AFS_UCRED *acred)
 				attrs->va_nodeid = ip->i_number;
 #endif
 			}
+#endif
 		    }
 #endif /* AFS_LINUX22_ENV */
 		}
@@ -395,19 +342,19 @@ afs_VAttrToAS(register struct vcache *avc, register struct vattr *av,
     register int mask;
     mask = 0;
     AFS_STATCNT(afs_VAttrToAS);
-#if	defined(AFS_AIX_ENV)
+#if     defined(AFS_DARWIN80_ENV)
+    if (VATTR_IS_ACTIVE(av, va_mode)) {
+#elif	defined(AFS_AIX_ENV)
 /* Boy, was this machine dependent bogosity hard to swallow????.... */
     if (av->va_mode != -1) {
-#else
-#if	defined(AFS_LINUX22_ENV)
+#elif	defined(AFS_LINUX22_ENV)
     if (av->va_mask & ATTR_MODE) {
-#else
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#elif	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (av->va_mask & AT_MODE) {
+#elif	defined(AFS_XBSD_ENV)
+    if (av->va_mode != (mode_t)VNOVAL) {
 #else
     if (av->va_mode != ((unsigned short)-1)) {
-#endif
-#endif
 #endif
 	mask |= AFS_SETMODE;
 	as->UnixModeBits = av->va_mode & 0xffff;
@@ -417,54 +364,54 @@ afs_VAttrToAS(register struct vcache *avc, register struct vattr *av,
 	    ReleaseWriteLock(&avc->lock);
 	}
     }
-#if defined(AFS_LINUX22_ENV)
+#if     defined(AFS_DARWIN80_ENV)
+    if (VATTR_IS_ACTIVE(av, va_gid)) {
+#elif defined(AFS_LINUX22_ENV)
     if (av->va_mask & ATTR_GID) {
-#else
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (av->va_mask & AT_GID) {
-#else
-#if (defined(AFS_HPUX_ENV) || defined(AFS_SUN_ENV))
+#elif defined(AFS_HPUX_ENV)
 #if	defined(AFS_HPUX102_ENV)
     if (av->va_gid != GID_NO_CHANGE) {
 #else
     if (av->va_gid != ((unsigned short)-1)) {
 #endif
+#elif	defined(AFS_XBSD_ENV)
+    if (av->va_gid != (gid_t)VNOVAL) {
 #else
     if (av->va_gid != -1) {
-#endif
-#endif /* AFS_SUN5_ENV */
 #endif /* AFS_LINUX22_ENV */
 	mask |= AFS_SETGROUP;
 	as->Group = av->va_gid;
     }
-#if defined(AFS_LINUX22_ENV)
+#if     defined(AFS_DARWIN80_ENV)
+    if (VATTR_IS_ACTIVE(av, va_uid)) {
+#elif defined(AFS_LINUX22_ENV)
     if (av->va_mask & ATTR_UID) {
-#else
-#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (av->va_mask & AT_UID) {
-#else
-#if (defined(AFS_HPUX_ENV) || defined(AFS_SUN_ENV))
+#elif defined(AFS_HPUX_ENV)
 #if	defined(AFS_HPUX102_ENV)
     if (av->va_uid != UID_NO_CHANGE) {
+#elif	defined(AFS_XBSD_ENV)
+    if (av->va_uid != (uid_t)VNOVAL) {
 #else
     if (av->va_uid != ((unsigned short)-1)) {
 #endif
 #else
     if (av->va_uid != -1) {
-#endif
-#endif /* AFS_SUN5_ENV */
 #endif /* AFS_LINUX22_ENV */
 	mask |= AFS_SETOWNER;
 	as->Owner = av->va_uid;
     }
-#if	defined(AFS_LINUX22_ENV)
+#if     defined(AFS_DARWIN80_ENV)
+    if (VATTR_IS_ACTIVE(av, va_modify_time)) {
+#elif	defined(AFS_LINUX22_ENV)
     if (av->va_mask & ATTR_MTIME) {
-#else
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#elif	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (av->va_mask & AT_MTIME) {
 #else
     if (av->va_mtime.tv_sec != -1) {
-#endif
 #endif
 	mask |= AFS_SETMODTIME;
 #ifndef	AFS_SGI_ENV
@@ -532,43 +479,32 @@ afs_setattr(OSI_VC_DECL(avc), register struct vattr *attrs,
     if (flags & ATTR_LAZY)
 	goto done;
 #endif
-#ifndef AFS_DEC_ENV
     /* if file size has changed, we need write access, otherwise (e.g.
      * chmod) give it a shot; if it fails, we'll discard the status
      * info.
-     *
-     * Note that Ultrix actually defines ftruncate of a file you have open to
-     * be O.K., and does the proper access checks itself in the truncate
-     * path (unlike BSD or SUNOS), so we skip this check for Ultrix.
-     *
      */
-#if	defined(AFS_LINUX22_ENV)
+#if	defined(AFS_DARWIN80_ENV)
+    if (VATTR_IS_ACTIVE(attrs, va_data_size)) {
+#elif	defined(AFS_LINUX22_ENV)
     if (attrs->va_mask & ATTR_SIZE) {
-#else
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#elif	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (attrs->va_mask & AT_SIZE) {
-#else
-#ifdef	AFS_OSF_ENV
+#elif	defined(AFS_OSF_ENV) || defined(AFS_XBSD_ENV)
     if (attrs->va_size != VNOVAL) {
-#else
-#ifdef	AFS_AIX41_ENV
+#elif	defined(AFS_AIX41_ENV)
     if (attrs->va_size != -1) {
 #else
     if (attrs->va_size != ~0) {
-#endif
-#endif
-#endif
 #endif
 	if (!afs_AccessOK(avc, PRSFS_WRITE, &treq, DONT_CHECK_MODE_BITS)) {
 	    code = EACCES;
 	    goto done;
 	}
     }
-#endif
 
     afs_VAttrToAS(avc, attrs, &astat);	/* interpret request */
     code = 0;
-#if defined(AFS_SUN_ENV) || defined(AFS_ALPHA_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
     afs_BozonLock(&avc->pvnLock, avc);
 #endif
 #if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
@@ -580,18 +516,16 @@ afs_setattr(OSI_VC_DECL(avc), register struct vattr *attrs,
 #if defined(AFS_SGI_ENV)
     AFS_RWLOCK((vnode_t *) avc, VRWLOCK_WRITE);
 #endif
-#if	defined(AFS_LINUX22_ENV)
+#if	defined(AFS_DARWIN80_ENV)
+    if (VATTR_IS_ACTIVE(attrs, va_data_size)) {
+#elif	defined(AFS_LINUX22_ENV)
     if (attrs->va_mask & ATTR_SIZE) {
-#else
-#if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+#elif	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
     if (attrs->va_mask & AT_SIZE) {
-#else
-#if	defined(AFS_OSF_ENV)
+#elif	defined(AFS_OSF_ENV) || defined(AFS_XBSD_ENV)
     if (attrs->va_size != VNOVAL) {
-#else /* AFS_OSF_ENV */
+#else
     if (attrs->va_size != -1) {
-#endif
-#endif
 #endif
 	afs_size_t tsize = attrs->va_size;
 	ObtainWriteLock(&avc->lock, 128);
@@ -616,10 +550,6 @@ afs_setattr(OSI_VC_DECL(avc), register struct vattr *attrs,
 	ReleaseWriteLock(&avc->lock);
 	hzero(avc->flushDV);
 	osi_FlushText(avc);	/* do this after releasing all locks */
-#ifdef AFS_DEC_ENV
-	/* in case we changed the size here, propagate it to gp->g_size */
-	afs_gfshack((struct gnode *)avc);
-#endif
     }
     if (code == 0) {
 	ObtainSharedLock(&avc->lock, 16);	/* lock entry */
@@ -640,7 +570,7 @@ afs_setattr(OSI_VC_DECL(avc), register struct vattr *attrs,
 	avc->execsOrWriters--;
     }
 #endif
-#if defined(AFS_SUN_ENV) || defined(AFS_OSF_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_BOZONLOCK_ENV
     afs_BozonUnlock(&avc->pvnLock, avc);
 #endif
 #if defined(AFS_SGI_ENV)
