@@ -18,7 +18,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vol/vol-info.c,v 1.18.2.1 2004/08/25 07:14:19 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vol/vol-info.c,v 1.22.4.1 2007/10/31 04:09:45 shadow Exp $");
 
 #include <ctype.h>
 #include <errno.h>
@@ -217,7 +217,7 @@ AttachVolume(struct DiskPartition * dp, char *volname,
 
 
 static int
-handleit(struct cmd_syndesc *as)
+handleit(struct cmd_syndesc *as, void *arock)
 {
     register struct cmd_item *ti;
     int err = 0;
@@ -256,7 +256,7 @@ handleit(struct cmd_syndesc *as)
     if ((ti = as->parms[5].items))
 	partName = ti->data;
     if ((ti = as->parms[6].items))
-	volumeId = atoi(ti->data);
+	volumeId = strtoul(ti->data, NULL, 10);
     if (as->parms[7].items)
 	dheader = 1;
     else
@@ -499,7 +499,7 @@ HandleVolume(struct DiskPartition *dp, char *name)
 		printf("\tVolId\t= %u\n", header.id);
 	    }
 
-	    IH_INIT(ih, dp->device, header.id, header.volumeInfo);
+	    IH_INIT(ih, dp->device, header.parent, header.volumeInfo);
 	    fdP = IH_OPEN(ih);
 	    if (fdP == NULL) {
 		perror("opening volume info");
@@ -519,7 +519,7 @@ HandleVolume(struct DiskPartition *dp, char *name)
 		       PrintInode(NULL, header.volumeInfo), code);
 	    }
 
-	    IH_INIT(ih, dp->device, header.id, header.smallVnodeIndex);
+	    IH_INIT(ih, dp->device, header.parent, header.smallVnodeIndex);
 	    fdP = IH_OPEN(ih);
 	    if (fdP == NULL) {
 		perror("opening small vnode index");
@@ -538,7 +538,7 @@ HandleVolume(struct DiskPartition *dp, char *name)
 		       PrintInode(NULL, header.smallVnodeIndex), code);
 	    }
 
-	    IH_INIT(ih, dp->device, header.id, header.largeVnodeIndex);
+	    IH_INIT(ih, dp->device, header.parent, header.largeVnodeIndex);
 	    fdP = IH_OPEN(ih);
 	    if (fdP == NULL) {
 		perror("opening large vnode index");
@@ -560,7 +560,7 @@ HandleVolume(struct DiskPartition *dp, char *name)
 #endif
 	    }
 #ifdef AFS_NAMEI_ENV
-	    IH_INIT(ih, dp->device, header.id, header.linkTable);
+	    IH_INIT(ih, dp->device, header.parent, header.linkTable);
 	    fdP = IH_OPEN(ih);
 	    if (fdP == NULL) {
 		perror("opening link table index");
@@ -587,7 +587,7 @@ HandleVolume(struct DiskPartition *dp, char *name)
 	vp = AttachVolume(dp, name, &header);
 	if (!vp) {
 	    printf("Volinfo: Error attaching volume header %s\n", name);
-	    exit(1);
+	    return;
 	}
     }
     PrintHeader(vp);
@@ -612,6 +612,8 @@ HandleVolume(struct DiskPartition *dp, char *name)
 	       Vauxsize_k, Vvnodesize_k, totvolsize, totvolsize - Vdiskused,
 	       V_name(vp));
     }
+    free(vp->header);
+    free(vp);
 }
 
 int
@@ -620,7 +622,7 @@ main(int argc, char **argv)
     register struct cmd_syndesc *ts;
     afs_int32 code;
 
-    ts = cmd_CreateSyntax(NULL, handleit, 0, "Dump volume's internal state");
+    ts = cmd_CreateSyntax(NULL, handleit, NULL, "Dump volume's internal state");
     cmd_AddParm(ts, "-online", CMD_FLAG, CMD_OPTIONAL,
 		"Get info from running fileserver");
     cmd_AddParm(ts, "-vnode", CMD_FLAG, CMD_OPTIONAL, "Dump vnode info");
@@ -690,6 +692,7 @@ PrintHeader(register Volume * vp)
 	 V_dayUse(vp), V_weekUse(vp)[0], V_weekUse(vp)[1], V_weekUse(vp)[2],
 	 V_weekUse(vp)[3], V_weekUse(vp)[4], V_weekUse(vp)[5],
 	 V_weekUse(vp)[6], date(V_dayUseDate(vp)));
+    printf("volUpdateCounter = %u\n", V_volUpCounter(vp));
 }
 
 /* GetFileInfo
